use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.006002
use Test::Spelling 0.12;
use Pod::Wordlist;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib  ) );
__DATA__
postorder
rjbs
Signes
subclasses
timestamps
tuples
David
Golden
dagolden
Steinbrunner
dsteinbrunner
Gian
Piero
Carrubba
gpiero
Graham
Knop
haarg
Ricardo
Toby
Inkster
tobyink
lib
Path
Iterator
Rule
PIR
